package RPM::Source::Convert::Plugins::FedoraFontPkg2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory 0.014;
use RPM::Source::Tools::Dependency::Filter;
use Source::Shared::Resource::DistroMap;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Resource::Datapath 0.001;
use RPM::Source::Transformation::Factory::DependencyFilter;
use RPM::Source::Transformation::Factory::TryRPMBuildPrep 0.002;
use RPM::Source::Convert::Plugins::SRPMReset;
use RPM::Source::Convert::Tools::LuaFedoraCommon;
use RPM::Source::Convert::Tools::Fonts;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

# fake section to be processed in &convert_font_pkg
# a method to protect multisection macro from killing as the part of %clean section
use RPM::Source::SpecSection;
# old fc macros
$RPM::Source::SpecSection::SECTION_MACROS{'_font_pkg'}=1;
# new fc macros
# see https://docs.fedoraproject.org/en-US/packaging-guidelines/FontsPolicy/
$RPM::Source::SpecSection::SECTION_MACROS{'fontpkg'}=1;

our $verbose=0;

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Fedora Font dependencies',
    -description => 'convert Fedora specific Font dependencies',
    -class => 'RPM::Source::Tools::Dependency::Filter::FedoraFont',
    -tags => SPEC_TAG_SET_REQUIRES | SPEC_TAG_SET_CONFLICTS | SPEC_TAG_PROVIDES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-enable_fedora_font_old_posthook => 0,
	});
}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_REINIT,
	    -name => 'fedora2alt_reinit_fontpkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_fontpkg_reinit),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_REINIT,
	    -name => 'fedora2alt_reinit_old_font_pkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_old_font_pkg_reinit),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_POSTHOOKS,
	    # TODO: insert_first => 1 ? it seems there is no need
	    -name => 'fedora2alt_posthook_old_font_pkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_old_font_pkg_posthook),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_TRY_RPMBUILD_PREP_PROCESS,
	    -name => 'fedora2alt_post_try_rpmbuild_bp_fontpkg',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_post_try_rpmbuild_bp_fontpkg);
}

# calculate fontpkgname
sub __suffix_for {
    my ($macros, $name, $suffix)=@_;
    return $suffix if defined ($suffix) and $macros->get_raw($name.$suffix);
    return '0' if (!defined($suffix) or $suffix eq '') and $macros->get_raw($name.'0');
    return '' if $macros->get_raw($name);
    return $suffix;
}

# Create a single %package section for a fonts subpackage
sub _singlefontpkg {
    my ($spec,$macros,$resourceobj,$suffix, $forcemain, $forcesub)=@_;
    my $is_sub = (! $forcemain && ($forcesub || $suffix));
    warn '_singlefontpkg: forcemain=',($forcemain?$forcemain:'false'),' forcesub=',($forcesub?$forcesub:'false'),' suffix=',(defined $suffix?$suffix:''), ' is_sub=', $is_sub,"\n" if $verbose >1;
    my $license_suffix =&__suffix_for($macros,'fontlicense',$suffix);
    my $nameprefix = $is_sub ? "%package     -n " : "Name:           ";
    my $fontpkgname = &RPM::Source::Convert::Tools::Fonts::__get_fontpkgname($spec,$macros,$resourceobj,$suffix);
    my $out= $nameprefix . $fontpkgname . "\n" .
      'Summary:        %{fontsummary'.&__suffix_for($macros,'fontsummary',$suffix)."}\n";
    $out.=
      'License:        %{fontlicense'.$license_suffix."}\n" if $macros->get_raw('fontlicense'.$license_suffix);
    $out.=
      "BuildArch:      noarch\n" .
      "BuildRequires:  rpm-build-fonts\n" .
      '%{?fontpkgheader'.&__suffix_for($macros,'fontpkgheader',$suffix)."}\n" .
      '%description'.($is_sub ? ' -n '.$fontpkgname:'')."\n" .
      '%{?fontdescription'.&__suffix_for($macros,'fontdescription',$suffix)."}\n";
    return $out;
}

# Create a font (sub)metapackage header
sub _fontmetapkg {
    my ($spec,$macros,$resourceobj,$name,$summary,$description,$suffixes)=@_;
    if ($name) {
	$name = '-n '.$name;
    } else {
	#my $fontpkgname = &RPM::Source::Convert::Tools::Fonts::__get_fontpkgname($spec,$macros,$resourceobj,'');
	#$fontpkgname=~s,-fonts$,,;
	#$name='-n '.$fontpkgname.'-fonts-all';
	$name='all';
    }
    unless ($summary) {
	$summary = "All the font packages, generated from %{name}";
    }
    unless ($description) {
	$description = "This meta-package installs all the font packages, generated from the %{name} source package.";
    }
    $description = &RPM::Source::Convert::Tools::LuaFedoraCommon::wordwrap($description);
    my $out='';
    # not supported in alt
    #my $Requires='Requires(meta)';
    my $Requires='Requires';
    $out.=
	'%package   ' . $name    . "\n" .
	'Summary:   ' . $summary . "\n";
    if (!defined $suffixes or $suffixes eq '') {
	foreach my $suffix ($macros->get_all_suffixes_for('fontfamily')) {
	    $out.= $Requires.':  ' . &RPM::Source::Convert::Tools::Fonts::__get_fontpkgname($spec,$macros,$resourceobj,$suffix) . " = %EVR\n";
	}
    } else {
	my %known_suffix=map{$_=>1} $macros->get_all_suffixes_for('fontfamily');
	foreach my $suffix (split(/\s*[.,]+\s*/,$suffixes)) {
	    if ($known_suffix{$suffix}) {
		$out.= $Requires.':  ' . &RPM::Source::Convert::Tools::Fonts::__get_fontpkgname($spec,$macros,$resourceobj,$suffix) . " = %EVR\n";
	    }
	}
    }
    $out.=
    "BuildArch: noarch\n" .
    '%description ' . $name . "\n" .
    $description            . "\n" .
    '%files '      . $name . "\n\n";
    return $out;
}

sub __process_all {
    my $spec=shift;
    my $macros=shift;
    my $resourceobj=shift;
    my $suffix=shift;
    my $processall=shift;
    my $process_sub=shift;
    my @args=@_;
    if ($processall) {
	my $accumulator='';
	foreach my $suffix ($macros->get_all_suffixes_for('fontfamily')) {
	    $accumulator .= &$process_sub($spec,$macros,$resourceobj,$suffix, @args);
	}
	return $accumulator;
    } else {
	&$process_sub($spec,$macros,$resourceobj, $suffix, @args);
    }
}

my %_font_rpm_build_handlers=(
    'fontbuild' => \&RPM::Source::Convert::Tools::Fonts::__fontbuild,
    'fontinstall' => \&RPM::Source::Convert::Tools::Fonts::__fontinstall,
    'fontcheck' => \&RPM::Source::Convert::Tools::Fonts::__fontcheck,
);

sub opt_handler_fedora2alt_fontpkg_reinit {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    my $buildreq=$resourceobj->buildreq;
    # first search for %fontpkg
    my $flag_fontpkg_processed=0;
    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    my $macros=$spec->macros();
    my $source_name=&RPM::Source::Convert::Tools::Fonts::__get_old_name($spec,$macros,$resourceobj);
    if (!$macros->get_raw('name')) {
	$macros->set_raw('name',$source_name);
    }
    print STDERR 'DEBUG: fedora2alt_fontpkg_reinit: ',$source_name,"\n" if $verbose>2;
    my @fontdefs_pushdown;
    my $fontpkgname_is_used=0;
    my $_fontdir_is_used=0;
    foreach my $sec (@sections) {
	$sec->apply_body_and_header(sub {
	    local $_=shift;
	    # hack for now;
	    #s,^\%linuxtext,#linuxtext,;

	    if (/\%\{?fontpkgname\d*\b\}?/) {
		$fontpkgname_is_used=1;
		if (/^Source.*fontpkgname(\d*)/) {
		    #Source12:  60-%{fontpkgname2}.xml
		    my $suffix=$1;
		    my $fontpkgname = &RPM::Source::Convert::Tools::Fonts::__get_fontpkgname($spec,$macros,$resourceobj,$suffix);
		    s,\%\{?fontpkgname$suffix\}?,$fontpkgname,e;
		}
		return $_;
	    } elsif (/^\s*%global\s*font(?:s|confs|confngs|appstreams)\d*\s+\%\{?S/) {
		# %global fontconfs       %{SOURCE1}
		# %global fontconfngs       %{SOURCE10}
		push @fontdefs_pushdown, $_;
		return;
	    } elsif (/^\s*%wordwrap\s+(?:-v\s+(\S+))\s*$/) {
		# hack for now; move to Fedora2ALT and use LuaCommon::wordwrap
		my $macrosname=$1;
		$macrosname//='_description';
		return '%'.$macrosname."\n";
	    # } elsif (/^\s*\%\{??fontdescription0\}?/) {
	    # 	if (!$macros->get_raw('fontdescription0') and $macros->get_raw('fontdescription')) {
	    # 	    s,fontdescription0,fontdescription,g;
	    # 	}
	    # 	return $_;
	    } elsif (/^\s*\%\{?fontfiles\b\}?/) {
		print STDERR 'DEBUG: fedora2alt_fontpkg_reinit activated: %fontfiles detected: ',$_ if $verbose>1;
		$flag_fontpkg_processed=1;
		$fontpkgname_is_used=1;
		chomp;
		s/^\s*\%\{?fontfiles\}?\s*//;
		my $arg_suffix='';
		my ($arg_a);
		while (s/^(\S+)\s*//) {
		    my $opt=$1;
		    if ($opt eq '-z') {
			s/^(\S+)\s*//;
			$arg_suffix=$1;
		    } elsif ($opt eq '-a') {
			$arg_a=1;
		    } elsif ($opt eq '-v') {
			# ignore
		    } else {
			die "Oops! unsupported arg: $opt";
		    }
		}
		my $arg_processall=($arg_a && ! $arg_suffix);
		return &__process_all(
		    $spec,
		    $macros,
		    $resourceobj,
		    $arg_suffix,
		    $arg_processall,
		    \&RPM::Source::Convert::Tools::Fonts::_fontfiles,
		    );
	    } elsif (/\%\{?_fontdir\b\}?/) {
		# side effect; %_fontdir is set in Fonts::_fontfiles
		my $fontdir=$macros->get_raw('_fontdir');
		if ($fontdir) {
		    s,\%\{?_fontdir\b\}?,$fontdir,;
		} else {
		    # if $fontdir is not defined,
		    # it means %_fontdir is used in %install
		    # so we need to get fontsubdir from distromap and
		    # define _fontdir
		    # see steinberg-bravura,steinberg-petaluma
		    $_fontdir_is_used=1;
		}
		return $_;
	    } elsif (/^\s*\%\{?fontpkg\b\}?/) {
		print STDERR 'DEBUG: fedora2alt_fontpkg_reinit activated: %fontpkg detected: ',$_ if $verbose>1;
		$flag_fontpkg_processed=1;
		$fontpkgname_is_used=1;
		chomp;
		s/^\s*\%\{?fontpkg\}?\s*//;
		#my @out='%package'."\n";
		my $fontpkg_arg_suffix='';
		my ($fontpkg_arg_a,$fontpkg_arg_forcemain,$fontpkg_arg_forcesub);
		while (s/^(\S+)\s*//) {
		    my $opt=$1;
		    if ($opt eq '-z') {
			s/^(\S+)\s*//;
			$fontpkg_arg_suffix=$1;
		    } elsif ($opt eq '-a') {
			$fontpkg_arg_a=1;
		    } elsif ($opt eq '-v') {
			# ignore
		    } elsif ($opt eq '-m') {
			$fontpkg_arg_forcemain=1;
		    } elsif ($opt eq '-s') {
			$fontpkg_arg_forcesub=1;
		    } else {
			die "Oops! unsupported arg: $opt";
		    }
		}
		my $fontpkg_arg_processall=($fontpkg_arg_a && ! $fontpkg_arg_suffix);
		return &__process_all(
		    $spec,
		    $macros,
		    $resourceobj,
		    $fontpkg_arg_suffix,
		    $fontpkg_arg_processall,
		    \&_singlefontpkg,
		    $fontpkg_arg_forcemain,
		    $fontpkg_arg_forcesub,
		    );
	    } elsif (/^\s*\%\{?fontmetapkg\b\}?/) {
		print STDERR 'DEBUG: fedora2alt_fontpkg_reinit activated: %fontmetapkg detected: ',$_ if $verbose>1;
		$flag_fontpkg_processed=1;
		$fontpkgname_is_used=1;
		chomp;
		s/^\s*\%\{?fontmetapkg\}?\s*//;
		#my @out='%package'."\n";
		my ($fontpkg_arg_zlist,$fontpkg_arg_s,$fontpkg_arg_d,$fontpkg_arg_n);
		while (s/^(\S+)\s*//) {
		    my $opt=$1;
		    if ($opt eq '-z') {
			s/^(\S+)\s*//;
			$fontpkg_arg_zlist=$1;
		    } elsif ($opt eq '-s') {
			s/^(\S+)\s*//;
			$fontpkg_arg_s=$1;
		    } elsif ($opt eq '-d') {
			s/^(\S+)\s*//;
			$fontpkg_arg_d=$1;
		    } elsif ($opt eq '-n') {
			s/^(\S+)\s*//;
			$fontpkg_arg_n=$1;
		    } elsif ($opt eq '-v') {
			# ignore
		    } else {
			die "Oops! unsupported arg: $opt";
		    }
		}
		return &_fontmetapkg(
		    $spec,
		    $macros,
		    $resourceobj,
		    $fontpkg_arg_n,
		    $fontpkg_arg_s,
		    $fontpkg_arg_d,
		    $fontpkg_arg_zlist,
		    );
	    } else {
		#print STDERR "DEBUG: fedora2alt_fontpkg_reinit not found in: ",$_ if $verbose>8;
		return $_;
	    }
			 }, '%fontpkg converter');
    }

    if ($fontpkgname_is_used) {
	if (!$macros->get_raw('fontpkgname')) {
	    $macros->set_raw('fontpkgname',$source_name);
	    $spec->main_section->unshift_body('%define fontpkgname '.$source_name."\n");
	}
    }
    if ($flag_fontpkg_processed) {
        print STDERR "opt_handler_fedora2alt_fontpkg_reinit: -srpm_reset_speclist_after_reinit => 1\n" if $verbose>1;
	$opt->{-srpm_reset_speclist_after_reinit}=1;
	$buildreq->add_buildreq('rpm-build-fedora-compat-fonts');
	$buildreq->add_pre_buildreq('rpm-macros-fedora-compat','rpm-macros-fonts');#,'rpm-macros-fontpackages'
	$opt->{-opt_try_rpmbuild_prep}=1;
	unless ($resourceobj->datapath_get_name()) {
	    # due to fontforge, the name is not set in prepare_resource; set it forcefully
	    $resourceobj->datapath_set_name($source_name);
	    print STDERR 'FedoraFontPkg2ALT: datapath_set_name to ',$source_name,"\n" if $verbose;
	}
	# handle _fontdir for new fonts only:
	# for old fedora fonts _fontdir is handled there
	if ($_fontdir_is_used) {
	    my @mapping=$distromap->multimap_source_origin2dest($source_name);
	    print STDERR 'FedoraFontPkg2ALT: multiple distromap for ',$source_name,': ',join(' ',@mapping),"\n" if @mapping>1;
	    print STDERR 'FedoraFontPkg2ALT: no distromap for ',$source_name,"\n" if @mapping==0;
	    if (1==@mapping) {
		if ($mapping[0] =~ /^fonts-([a-zA-Z0-9]+)-(\S+)$/) {
		    push @fontdefs_pushdown, '%define _fontdir %_fontsdir/'.$1.'/'.$2."\n";
		} else {
		    print STDERR 'FedoraFontPkg2ALT: not a font distromap for ',$source_name,': ',join(' ',@mapping),"\n";
		}
	    }
	}
    }
    if (@fontdefs_pushdown>0) {
	$spec->add_section('prep')->unshift_body(@fontdefs_pushdown);
	$opt->{-srpm_reset_macros_after_reinit}=1;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_fedora2alt_post_try_rpmbuild_bp_fontpkg {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK unless $opt->{-opt_try_rpmbuild_prep};
    my $buildsec=$spec->get_section('build');
    my $instsec=$spec->get_section('install');
    unless ($buildsec and $instsec) {
	warn 'FedoraFontPkg2ALT: fatal: %build or %install not found',"\n";
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    my $setupdir=$opt->{-try_rpmbuild_prep_sourcetree};
    unless (defined $setupdir) {
	warn 'FedoraFontPkg2ALT: fatal: setuptree not found',"\n";
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    my $macros=$spec->macros();
    my @all_suffixes=$macros->get_all_suffixes_for('fontfamily');
    my $_font_rpm_build_macro=sub {
	# append #HIDDEN% to %fontbuild will allow smooth rpmbuild -bb
	if (s/^(?:#HIDDEN\%)?\%(font(?:build|install|check))//) {
	    my $fmname=$1;
	    my $handler=$_font_rpm_build_handlers{$fmname};
	    my ($opt_a,$opt_z);
	    $opt_a =1 if s/-a//;
	    $opt_z =$1 if s/-z\s*(\d+)//;
	    die 'FedoraFontPkg2ALT: %',$fmname,': unknown opts: ',$_ if ! /^\s*$/;
	    # accumulate -a
	    $_='';
	    if ($opt_a) {
		foreach my $suffix (@all_suffixes) {
		    print STDERR 'DEBUG: fedora2alt_fontpkg_reinit: processing %',$fmname,' -a: suffix=',$suffix,"\n" if $verbose>2;
		    $_ .= &$handler($spec,$macros,$resourceobj,$suffix,$setupdir);
		}
	    } else {
		print STDERR 'DEBUG: fedora2alt_fontpkg_reinit: processing %',$fmname,"\n" if $verbose>2;
		$_ .= &$handler($spec,$macros,$resourceobj,(defined($opt_z)?$opt_z:''),$setupdir);
	    }
	}
    };
    $buildsec->map_body($_font_rpm_build_macro);
    $instsec->map_body($_font_rpm_build_macro);
    my $checksec=$spec->get_section('check');
    $checksec->map_body($_font_rpm_build_macro) if $checksec;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

#---------------------------------------------------------------------------------------

sub opt_handler_fedora2alt_old_font_pkg_posthook {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK
	if not $opt->{-enable_fedora_font_old_posthook};

    print STDERR "opt_handler_fedora2alt_font_pkg_posthook: start\n" if $verbose>1;

    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	} elsif ($t eq 'build') {
	} elsif ($t eq 'install') {
	    if (not $spec->{'CONVERT::FEDORA::NO_MOVE_FONTS'}) {
		$sec->push_body(q!# generic fedora font import transformations
# move fonts to corresponding subdirs if any
for fontpatt in OTF TTF TTC otf ttf ttc woff woff2 WOFF WOFF2 otb OTB pcf pcf.gz bdf afm pfa pfb; do
    case "$fontpatt" in 
	pcf*|bdf*|otb|OTB) type=bitmap;;
	tt*|TT*) type=ttf;;
	otf|OTF) type=otf;;
	afm*|pf*) type=type1;;
	woff|WOFF) type=woff;;
	woff2|WOFF2) type=woff2;;
    esac
    find $RPM_BUILD_ROOT/usr/share/fonts -type f -name '*.'$fontpatt | while read i; do
	j=`echo "$i" | sed -e s,/usr/share/fonts/,/usr/share/fonts/$type/,`;
	install -Dm644 "$i" "$j";
	rm -f "$i";
	olddir=`dirname "$i"`;
	mv -f "$olddir"/{encodings.dir,fonts.{dir,scale,alias}} `dirname "$j"`/ 2>/dev/null ||:
	rmdir -p "$olddir" 2>/dev/null ||:
    done
done
!);
	    }
	    if (not $spec->{'CONVERT::FEDORA::NO_RELINK_FONTCATALOG'}) {
		$sec->push_body(q!# kill invalid catalogue links
if [ -d $RPM_BUILD_ROOT/etc/X11/fontpath.d ]; then
    find -L $RPM_BUILD_ROOT/etc/X11/fontpath.d -type l -print -delete ||:
    # relink catalogue
    find $RPM_BUILD_ROOT/usr/share/fonts -name fonts.dir | while read i; do
	pri=10;
	j=`echo $i | sed -e s,$RPM_BUILD_ROOT/usr/share/fonts/,,`; type=${j%%%%/*}; 
	pre_stem=${j##$type/}; stem=`dirname $pre_stem|sed -e s,/,-,g`;
	case "$type" in 
	    bitmap) pri=10;;
	    ttf|ttf) pri=50;;
	    type1) pri=40;;
	esac
	ln -s /usr/share/fonts/$j $RPM_BUILD_ROOT/etc/X11/fontpath.d/"$stem:pri=$pri"
    done ||:
fi
!);
	    }
	} elsif ($t eq 'files') {
	    $sec->subst_body(qr'\%\{_fontdir\}','%{_fontsdir}/*/%{_fontstem}');
	    $sec->map_body(sub {
		if (m!^(.*)\%\{catalogue\}/(\S+)\s*$!) {
		    my $pathbase=$1;
		    my $stem=$2;
		    $stem=~s/-fonts$//;
		    $_=$pathbase.'%{catalogue}/'.$stem."*\n" unless /\*\s*$/;
		}
	    });
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_fedora2alt_old_font_pkg_reinit {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    # first search for %_font_pkg
    my $flag_font_pkg_processed=0;
    my @sections=$spec->get_sections();
    my $mainsec=$sections[0];
    unless ($mainsec->match_body(qr'Build(?:Pre)?Req.*fontpackages-devel')) {
	print STDERR "DEBUG: fedora2alt_old_font_pkg_reinit: fontpackages-devel not found, exiting...\n" if $verbose>2;
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
    }

    my $source_name=$spec->macros()->macro_subst('%name');
    print STDERR 'DEBUG: fedora2alt_font_pkg_reinit: ',$source_name,"\n" if $verbose>2;

    foreach my $sec (@sections) {
	$sec->apply_body_and_header(sub {
	    local $_=shift;
	    if (m!^\s*\%ghost.*\%\{?_fontdir\}?/.uuid!) {
		#%ghost %attr(644, root, root) %{_fontdir}/.uuid
		return '';
	    } elsif (! /^\s*\%\{?_font_pkg\}?/) {
		#print STDERR "DEBUG: fedora2alt_font_pkg_reinit not found in: ",$_ if $verbose>8;
		return $_;
	    } else {
		print STDERR 'DEBUG: fedora2alt_font_pkg_reinit activated: %_font_pkg detected: ',$_ if $verbose>1;
		$flag_font_pkg_processed=1;
		chomp;
		s/^\s*\%\{?_font_pkg\}?\s*//;
		my @out='%files'."\n";
		@out='%files -n fonts-ttf-unifont'."\n" if $source_name eq 'unifont';
		my @filepattern;
		while (s/^(\S+)\s*//) {
		    if ($1 eq '-f') {
			s/^(\S+)\s*//;
			push @out, '%{_fontconfig_templatedir}/'."$1\n";
			push @out, '%config(noreplace) %{_fontconfig_confdir}/'."$1\n";
		    } elsif ($1 eq '-n') {
			s/^(\S+)\s*//;
			my $fontstem=$1;
			my $fcname=lc($fontstem); # in fc they lc :) see mathjax
			my $pkgsec=$spec->get_section('package',"-n $fcname");
			if ($pkgsec) {
			    print STDERR "fedora2alt_reinit_old_font_pkg: found %package -n $fcname\n" if $verbose>1;
			} else {
			    print STDERR "fedora2alt_reinit_old_font_pkg: not found %package -n $fcname\n" if $verbose>1;
			    $pkgsec=$spec->get_section('package',"-n $fcname-fonts");
			    if ($pkgsec) {
				$fcname .= '-fonts';
				print STDERR "fedora2alt_reinit_old_font_pkg: found %package -n $fcname-fonts\n" if $verbose>1;
			    } else {
				print STDERR "fedora2alt_reinit_old_font_pkg: not found %package -n $fcname-fonts\n" if $verbose>1;
				$pkgsec=$spec->get_section('package',"$fcname-fonts");
				if ($pkgsec) {
				    print STDERR "fedora2alt_reinit_old_font_pkg: found %package $fcname-fonts\n" if $verbose>1;
				    $fcname = $spec->macros->macro_subst('%name').'-'.$fcname.'-fonts';
				}
			    }
			}
			# -n can be a real package name as in ht-alegreya-fonts
			if (not $pkgsec) {
			    $fcname='%{?fontname:%fontname}%{!?fontname:%name}-'.$fontstem.'-fonts';
			    print STDERR "fedora2alt_reinit_old_font_pkg: fontstem=$fontstem fcname=$fcname\n" if $verbose>1;
			    # mplus-fonts
			    $fcname=$fontstem.'-fonts' if ($fontstem =~/fontname/ and $fcname!~/-fonts$/);
			}
			my @mapname=$distromap->multimap_binary_origin2dest($spec->macros()->macro_subst($fcname));
			$mapname[0]||=$fcname;
			$out[0] ='%files -n '.$mapname[0]."\n";
		    } else {
			push @filepattern, $1;
		    }
		}
		s/\s*$//;
		push @filepattern, $_ if $_;
		# does not work for multi packages
		#push @out, '%{_fontsdir}/*'."\n";
		push @out,
		    # or else dir will be unowned
		    '%dir %{_fontsdir}/*/%{_fontstem}/'."\n",
			map { '%{_fontsdir}/*/%{_fontstem}/'.$_."\n"}
			map {$_ eq '%{_fontdir}' ? '*': $_} @filepattern;
		return @out;
	    }
			 }, '%_font_pkg converter');
    }

    if ($flag_font_pkg_processed) {
        print STDERR "opt_handler_fedora2alt_reinit_old_font_pkg: -enable_fedora_font_old_posthook => 1\n" if $verbose>1;
	$opt->{-srpm_reset_speclist_after_reinit}=1;
	$opt->{-enable_fedora_font_old_posthook}=1;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;

package RPM::Source::Tools::Dependency::Filter::FedoraFont;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $pkgname=$dep->{-name};
    return $dep if $pkgname!~/^font\(:lang=(.+)\)$/;
    my $langreq=$1;
    $dep->{-name}='fontlang('.$langreq.')';
    return $dep;
}

1;

__END__
