package RPM::Source::Convert::Plugins::FedoraForgeMeta2ALT;
use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Factory 0.012;
use RPM::Source::Convert::Tools::ForgeMeta;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Convert::Plugins::SRPMReset;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

#use RPM::Source::SpecSection;
#$RPM::Source::SpecSection::SECTION_MACROS{'forgemeta'}=1;

our $verbose=0;
our $opt_forgeautosetup_patch_prune;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_REINIT,
	    -name => 'fedora2alt_reinit_forgemeta',
	    -opt_handler_sub=>\&opt_handler_fedora2alt_forgemeta_reinit);
}

sub opt_handler_fedora2alt_forgemeta_reinit {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    #my @sections=$spec->get_sections();
    #my $mainsec=$sections[0];
    my $macros=$spec->macros();
    my $flag_forgemeta_found=0;
    my $mainsec=$spec->main_section;
    my %forgeurl_suffix;
    $mainsec->visit_body(
	sub {
	    local $_=shift;
	    if (/\%\{?forgemeta(?:\}|\s|$)/) {
		$flag_forgemeta_found=1;
	    } elsif (/(?<!%)\%(?:define|global)\s+forgeurl(\d*)\s/) {
		$forgeurl_suffix{$1}=1;
	    }
	});
    return RPM::Source::TransformContainer::TRANSFORMATION_OK unless $flag_forgemeta_found;
    unless (%forgeurl_suffix) {
	print STDERR 'FedoraForgeMeta2ALT: %forgemeta found, but not %forgeurl(s). Transformation failed.',"\n";
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }
    my $has_extra_suffix=0;
    foreach my $key (keys(%forgeurl_suffix)) {
	$has_extra_suffix=1 if $key;
    }
    if ($has_extra_suffix) {
	print STDERR 'FedoraForgeMeta2ALT: %forgemeta found, but with extra suffixes. Transformation failed.',"\n";
	return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
    }

    $mainsec->apply_body(
	sub {
	    local $_=shift;
	    if (/\%\{?forgemeta(?:\}|\s|$)/) {
		my ($silent,$u_opt,$a_opt);
		my $suffix='';
		s,^.*\%\{?forgemeta\}?\s*,,;
		while (s,^-(\w+)\s*,,) {
		    my $opts=$1;
		    foreach my $opt (split (//, $opts)) {
			if ($opt eq 'v' or $opt eq 'i') {
			    # ignore
			} elsif ($opt eq 's') {
			    $silent=1;
			} elsif ($opt eq 'z') {
			    $suffix=$1 if s,^(\d+)\s*,,;
			} elsif ($opt eq 'a') {
			    warn '%forgemeta -a: not supported yet.';
			    $a_opt=1;
			    return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE;
			} elsif ($opt eq 'u') {
			    s,\s*$,,;
			    $u_opt=$_;
			}
		    }
		}
		my @generated_meta;
		unless ($a_opt) {
		    &RPM::Source::Convert::Tools::ForgeMeta::meta($macros,$suffix,$silent,$u_opt);
		    push @generated_meta, &RPM::Source::Convert::Tools::ForgeMeta::forgemeta_defines($macros,$suffix);
		} else {
		    # TODO
		    die 'oops';
		}
		return (
		    "# FedoraForgeMeta2ALT: generated meta\n",
		    @generated_meta,
		    "# FedoraForgeMeta2ALT: end generated meta\n",
		    );
	    } else {
		#print STDERR "DEBUG: fedora2alt_forgemeta_reinit not found in: ",$_ if $verbose>8;
		return $_;
	    }
	}, '%forgemeta converter');

    #foreach my $sec (@sections) {
    # }
    my $prepsec=$spec->get_section('prep');
    if ($prepsec) {
	my $suffix='';
	$prepsec->apply_body(
	sub {
	    local $_=shift;
	    if (/\%\{?forgesetup\}?/) {
		return '%setup -q '.$macros->macro_subst('%forgesetupargs'.$suffix)."\n";
	    } elsif (s/^\s*\%\{?forgeautosetup\}?//) {
		my $suffix=$_;
		my $args=$macros->macro_subst('%forgesetupargs'.$suffix);
		if (defined $opt_forgeautosetup_patch_prune) {
		    #print STDERR "opt_handler_fedora2alt_forgemeta_reinit: detected opt_forgeautosetup_patch_prune\n" if $verbose>1;
		    unless ($args=~s,(-p\s*)\d+,$1.$opt_forgeautosetup_patch_prune,e or
			    $suffix=~s,(-p\s*)\d+,$1.$opt_forgeautosetup_patch_prune,e) {
			$args ='-p'.$opt_forgeautosetup_patch_prune.' '.$args;
			#print STDERR 'opt_handler_fedora2alt_forgemeta_reinit: added opt_forgeautosetup_patch_prune: args=',$args,"\n" if $verbose>1;
		    } else {
			#print STDERR 'opt_handler_fedora2alt_forgemeta_reinit: substituted opt_forgeautosetup_patch_prune: args=',$args,'suf=',$suffix,"\n" if $verbose>1;
		    }
		}
		return '%autosetup '.$args.$suffix;
	    } else {
		return $_;
	    }
	}, '%forgemeta prep converter');

    }

    print STDERR "opt_handler_fedora2alt_forgemeta_reinit: -srpm_reset_ s _after_reinit => 1\n" if $verbose>1;
    $opt->{-srpm_reset_sources_after_reinit}=1;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}


#---------------------------------------------------------------------------------------

1;

__END__
