package RPM::Source::Convert::Plugins::JPackage2ALT;

use strict;
use warnings;
use Carp;

use RPM::Vercmp;
use File::Basename;
use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use Source::Shared::Resource::DistroMap;
# after Generic2ALT; not to disturb the order
use RPM::Source::Convert::Plugins::DistroMap::Groups;
use RPM::Source::Convert::Plugins::Fedora2ALT;
use RPM::Source::Transformation::Factory::RaiseRelease;
use RPM::Source::Transformation::Factory::DependencyFilter;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

$RPM::Source::Convert::Plugins::DistroMap::Groups::default_group='Development/Java';

$RPM::Source::Transformation::Factory::RaiseRelease::nextrel='jppimport';
$RPM::Source::Transformation::Factory::RaiseRelease::defaultchangelog='- new jpp release';
&RPM::Source::Transformation::Factory::RaiseRelease::register_nextrel_method(
    -name => 'jppimport',
    -nextrel_method => \&nextrel_jppimport,
    -initrel_method => \&initrel_jppimport,
    );

our $verbose=0;
our $jvm='default';
our ($opt_no_compat);

our @LONGOPT=(
    "jvm=s" => \$jvm,
    "no-compat" => \$opt_no_compat,
    );

my %jvm2rpm=qw/
8	jpackage-1.8-compat
11	jpackage-11
generic	jpackage-generic-compat
default	jpackage-default
/;

my %jvm2rel=qw/
8	8
11	11
generic	11
default	11
/;

sub print_longopt {
    print "JPackage2ALT options:\n";
    print '	--jvm <'.join('|',sort(keys(%jvm2rpm)))."> JVM environment. Default is java $jvm.\n";
}

sub process_options {
    my ($self)=@_;
    if (not $jvm and not $jvm2rpm{$jvm}) {
	print "ERROR: invalid jvm option $jvm\n";
	exit (5);
    }
}

sub transformations {
    return
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'jpackage2alt_convert',
	    -opt_handler_sub=>\&opt_handler_jpp_specific_convert);
}

# TODO drop into distromap db
use RPM::Source::Convert::Plugins::DistroMap::Packages;
&init_plugin();
sub init_plugin {
    &RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Drop JVM dependencies',
    -description => 'Drop JVM dependencies',
    -class => 'RPM::Source::Tools::Dependency::Filter::JPackage2ALT::DropReqJava',
    -tags => SPEC_TAG_REQUIRES | SPEC_TAG_SUGGESTS | SPEC_TAG_RECOMMENDS,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
	);
    &RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Drop BR JVM dependencies',
    -description => 'Drop BuildReq JVM dependencies',
    -class => 'RPM::Source::Tools::Dependency::Filter::JPackage2ALT::DropBuildReqJava',
    -tags => SPEC_TAG_BUILD_REQUIRES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
	);
    &RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'Drop JVM dependencies',
    -description => 'Drop Req on javapackages-tools',
    -class => 'RPM::Source::Tools::Dependency::Filter::JPackage2ALT::DropReqJpackageUtils',
    -tags => SPEC_TAG_REQUIRES | SPEC_TAG_SUGGESTS | SPEC_TAG_RECOMMENDS,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
	);
    # BAD: if called after options, will overwrite option :( FIX in Groups.pm
    $RPM::Source::Convert::Plugins::DistroMap::Groups::default_group='Development/Java';
    map {$RPM::Source::Convert::Plugins::DistroMap::Groups::EXTRA_GROUP_PREMAP{$_} = 'Development/Java'} 
    (
	'Applications/Text',
	'Applications/Productivity',
	'Devel',
	'devel',
	'Development',
	'Development/Build Tools',
	'Development/Code Generators',
	'Development/Compilers',
	'Development/Framework',
	'Development/Languages', # ecj
	'Development/Libraries',
	'Development/Libraries/Application Frameworks',
	'Development/Testing',
	'Development/Tools',
	'Development/C',
	'Development/Frameworks', # richfaces
	'Internet/WWW/Dynamic Content', #tomcat
	'Internet/WWW/Servers', #jboss
	'Internet/WWW/Indexing/Search', # lucene
	'Applications/Internet', # apacheds
	'System/Logging', # log4j
	'System/Networking', # mina
	'System Environment/Applications', #tomcat
	'System Environment/Libraries',
	'Security/Cryptography',
	'Software Development/Quality Assurance',
	'Networking/Daemons', # jacarta-common-daemons
	'System/Boot',
	'Utilities', # jarjar
	'Text Processing/Markup/XML',
	'Text Processing/Markup/HTML',
	'Documentation', # jacorb
	'Development Documentation', # rome
	'Development/Debuggers', # findbugs
    );
}

sub opt_handler_jpp_specific_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $main_sec=$spec->get_section('package','');
    my $name=$main_sec->get_tag('Name');
    #deprecated?
    #$main_sec->unshift_body("BuildRequires: rpm-build-java-osgi\n") if $name=~/^eclipse/;

    my $has_jpp_environment_dependency;
    $main_sec->map_body(
	sub {
	    $has_jpp_environment_dependency=1
		if /^BuildRequires: jpackage-(?:[0-9]|default|generic|compat)/;
	});
    if (not $has_jpp_environment_dependency) {
	my $jppcompat=$jvm2rpm{$jvm};
	die "jvm not supported - no jpackage compat for ",$jvm unless $jppcompat or $opt_no_compat;
	if ($name !~ /^java-.*openjdk/) {
	    if ($opt_no_compat) {
		$main_sec->unshift_body("BuildRequires: java-devel-default\n");
	    } else {
		$resourceobj->buildreq->ignore_buildreq('java-devel-default');
		print STDERR 'info: using ',$jppcompat,' for jvm ',$jvm,"\n" if $verbose;
		$main_sec->unshift_body("BuildRequires: $jppcompat\n");
	    }
	}
	$main_sec->unshift_body("BuildRequires: /proc rpm-build-java\n");
    }

    $main_sec->visit_body(sub {
	print STDERR 'JPackage2ALT: found bcond_with: '.$1."\n" if (/^\%bcond_with\s+(\S+)\s*$/);
	print STDERR 'JPackage2ALT: found bcond_without: '.$1."\n" if (/^\%bcond_without\s+(\S+)\s*$/);
    });

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

### ---- nextrel ---------------------------------------------------

sub initrel_jppimport {
    my ($spec,undef)=@_;
    my $jpprel = $spec->get_section('package','')->get_tag('Release');
    die "jpp release is undefined" unless defined $jpprel;
    warn "initrel_jppimport:alt_ found! wrong tool is used? " if $jpprel=~/^alt/;
    return 'alt'.($1+1).$2 if $jpprel=~/^alt(\d+)[^_]*(_.+jpp.*)$/;

# $jppsubrel is used to keep branch of fedora or mandriva
    my $jppsubrel='';
# for new jpackage 5.0
    $jpprel=~s/\%\{dist\}//;
# for fedora-based imports (hack :()
    if ($jpprel=~/\%\{\?dist\}/) {
	$jpprel=~s/\%\{\?dist\}//;
	$jpprel=~s/jpp(\.\d+)?//;
	$jppsubrel=$1 if $1;
    }
    $jpprel=~s/jpp\d.\d$//;
    $jpprel=~s/.jpp5$//;
    $jpprel=~s/jpp//;

    my $altrel="alt1";
    my $newrel = $altrel.$jppsubrel."_".$jpprel.'jpp'.$jvm2rel{$jvm};
    return $newrel;
}

sub nextrel_jppimport {
    my ($rel,$spec,$parent)=@_;
    my $jppmain= $spec->get_section('package','');
    my $jppver = $jppmain->get_tag('Version');
    my $jpprel = $jppmain->get_tag('Release');
    warn "nextrel_jppimport:alt release $jpprel found! wrong tool is used? " if $jpprel=~/^alt/;
    die "jpp release is undefined" unless defined $jpprel;
# $jppsubrel is used to keep branch of fedora or mandriva
    my $jppsubrel='';
# for new jpackage 5.0
    $jpprel=~s/\%\{dist\}//;
# for fedora-based imports (hack :()
    if ($jpprel=~/\%\{\?dist\}/) {
	$jpprel=~s/\%\{\?dist\}//;
	$jpprel=~s/jpp(\.\d+)?//;
	$jppsubrel=$1 if $1;
    }
    $jpprel=~s/jpp\d.\d$//;
    $jpprel=~s/.jpp5$//;
    $jpprel=~s/jpp//;

    my $altrel="alt1";
    my $jvmrel=$jvm2rel{$jvm};
    if ($parent) {
	my $main_parent=$parent->main_section;
	$altrel = $main_parent->get_tag('Release');
	my $altver = $main_parent->get_tag('Version');
	my $vercmp = RPM::Vercmp::vercmp($altver,0,$jppver,0);
	chomp $vercmp;
	die 'Oops: undefined jvm2rel for ',$jvm unless $jvmrel;
	if ($altrel=~/^alt([^_]+)_(.+)jpp(\d+(?:\.\d)?)?$/) {
	    my $relcore = $1;
	    my $jppfrom = $2;
	    my $java= $3;
	    $java||=0;
	    my $subrel='';
	    $subrel=$1 if ($relcore=~s/(\.\d+)$//);
	    # hack -- cleanup (undefined in jpp) macro in alt release:
	    $relcore=~s/.?\%\w+//g;
	    $relcore=~s/.?\%\{\w+\}+//g;
	    print STDERR "new alt release: jpprel=$jpprel rel=$relcore subrel=$subrel jppfrom=$jppfrom jppsubrel=$jppsubrel $java\n" if $verbose>1;
	    if ($relcore=~/^0\./ or $vercmp != 0) {
		print STDERR "set rel=1\n" if $verbose>1;
		$relcore=1;
	    } elsif (($jppfrom eq $jpprel) and $subrel and $jppsubrel and $subrel < $jppsubrel) {
		# nothing to do
		print STDERR "nothing to do: new subrel is higher\n" if $verbose>1;
	    } elsif (not $subrel and &RPM::Vercmp::vercmp(0,$jpprel,0,$jppfrom) >0) {
		# nothing to do
		print STDERR "nothing to do: new jppfrom is higher\n" if $verbose>1;
	    } elsif (($jppfrom eq $jpprel) and not $subrel and $java and ($java < $jvmrel)) {
		# nothing to do
		print STDERR "nothing to do: new java is higher\n" if $verbose>1;
	    } else {
		$relcore=~/^(\d+)/;
		$relcore=$1+1;
		print STDERR "++rel=$relcore\n" if $verbose>1;
	    }
	    $altrel='alt'.$relcore;
	} else {
	    print STDERR "first alt release $altrel\n" if $verbose>1;
	    $altrel=~s/.?\%\w+//g;
	    $altrel=~s/.?\%\{\w+\}+//g;
	    if ($altrel=~/^alt0\./ or $vercmp != 0) {
		$altrel='alt1';
	    } elsif ($altrel=~/^alt(\d+)[\.a-zA-Z_]/) {
		$altrel='alt'.($1+1);
	    }
	}
    }
    my $newrel = $altrel.$jppsubrel."_".$jpprel.'jpp'.$jvmrel;
    return $newrel;
}

package RPM::Source::Tools::Dependency::Filter::JPackage2ALT::DropReqJava;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

# java-devel jce jsse ...  are hack around Conflicts: in jpackage-1.x-compat
#jce jsse 
my %javadep_skip = map {$_=>1} qw!
java-devel java-sdk java-headless
java-1.6.0-devel java-1.7.0-devel java-1.8.0-devel
java-1.6.0 java-1.7.0 java-1.8.0
jce jsse 
java-fonts!;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    $dep->{-name}=undef if $javadep_skip{$dep->{-name}};
    return $dep;
}

package RPM::Source::Tools::Dependency::Filter::JPackage2ALT::DropBuildReqJava;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

# java-devel jce jsse ...  are hack around Conflicts: in jpackage-1.x-compat
#jce jsse
my %javabr_skip = map {$_=>1} qw!
java-devel java-sdk java-headless
!;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    $dep->{-name}=undef if $javabr_skip{$dep->{-name}};
    return $dep;
}

# TODO: move to Fedora?
package RPM::Source::Tools::Dependency::Filter::JPackage2ALT::DropReqJpackageUtils;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;
sub apply {
    my ($self, $tagobj, $dep) = @_;
    my $depname=$dep->{-name};
    if ($depname eq 'jpackage-utils') {
	my $spec=$self->{-spec};
	# TODO check %files for %bindir, check %post, etc
	#$dep->{-name}=undef;
    } elsif ($depname eq 'javapackages-local' or $depname eq 'maven-local') {
	my $spec=$self->{-spec};
	# TODO check %files for %bindir, check %post, etc
	$dep->{-name}=undef unless $spec->main_section->get_tag('Name') eq 'javapackages-tools';
    }
    return $dep;
}

1;

__END__
