package RPM::Source::Convert::Plugins::SharedLibsPolicy;
use strict;
use warnings;

use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Resource::FlagLoader;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $VERSION=0.001;
our $verbose=0;

#------------------------------------------------------------------------------------
# db format: 4 colimns, 2 required, 2 optional:
# <srcname>     - required. example: armadillo
# <solib_major> - required. example: 9
# optional columns can be skipped or contain "-" sign
# <libname> - optional. used as (lib<libname>.so.<solib_major>). by default, srcname.
# <libpkg>  - optional. original name of the subpackage containing lib. by default, srcname.
#------------------------------------------------------------------------------------
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('shared_libs_convert.txt',FLAG_RESOURCE_LIST_AS_ARRAY);

sub transformations {
    return
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_PRECONVERT,
	    -name => 'sharedlibs_policy_convert',
	    -opt_handler_sub=>\&opt_handler_shared_libs_policy_convert),
	;
}

sub opt_handler_shared_libs_policy_convert {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    #my $macros=$spec->macros();
    my $mainsec=$spec->main_section;
    my $srcname=$spec->macros->get_raw('name');
    my $sharedlib_args=$resourceobj->get_flagdb('shared_libs_convert.txt');
    return RPM::Source::TransformContainer::TRANSFORMATION_OK unless $sharedlib_args;

    my ($solib_major,$libname,$libpkg)=@$sharedlib_args;
    $libname= $srcname=~s/^lib//r if not $libname or $libname eq '-';
    $libpkg='' if not defined($libpkg) or $libpkg eq '-';
    my $sharedlibname='lib'.$libname.$solib_major;
    die "Oops! already applied Shared Libs Policy: ",$srcname if $srcname eq $sharedlibname;

    my $donor_sec=$spec->get_section('files',$libpkg);
    my @donor_body=@{$donor_sec->get_bodyref()};
    # TODO __get_header_idx to be used someday
    my $donor_head=shift @donor_body;
    # test ...
    #%{_libdir}/libarmadillo.so.9*
    #%doc --no-dereference LICENSE.txt NOTICE.txt
    my $soname_glob_main=0;
    my $soname_glob_extra=0;
    my (@main_bin_body,@main_doc_body);
    my (@shared_lib_body,@shared_lib_gir_body,@shared_lib_bad_glob_body);
    my (@endspace_body);
    local $_;
    my $has_libprefix = $srcname=~ /^lib/;
    foreach (@donor_body) {
	if (m!^\%\{?_libdir\}?/lib${libname}\.so\.${solib_major}\s*$!) {
	    push @shared_lib_body, $_;
	    $soname_glob_main=1;
	} elsif (m!^\%\{?_libdir\}?/lib${libname}\.so\.${solib_major}\.\*\s*$!) {
	    push @shared_lib_body, $_;
	    $soname_glob_extra=1;
	} elsif (m!^\%\{?_libdir\}?/(?:lib${libname}|\*)\.so\.\d*\*\s*$!) {
	    push @shared_lib_bad_glob_body, $_;
	} elsif (m!^\%\{?_libdir\}?/\*\.so\.\d*\*\s*$!) {
	    push @shared_lib_bad_glob_body, $_;
	} elsif ($has_libprefix && m!^\%\{?_libdir\}?/\%\{?name\}?\.so\.\d*\*\s*$!) {
	    push @shared_lib_bad_glob_body, $_;
	# %lang(ja) %doc
	} elsif (m!^(?:\%lang\([-\w\d_\.]+\)\s+)?\%(?:doc|license)\s! or m!^\s*#! or m!^\s*$!) {
	    push @shared_lib_body, $_;
	#%exclude %{_pkgdocdir}/html
	} elsif (m!^\%exclude\s+\%\{?_(?:pkg)?docdir!) {
	    push @main_doc_body, $_;
	} elsif (m!^\%\{?_s?bindir\}?/!) {
	    push @main_bin_body, $_;
	} elsif (m!^\%\{?_mandir\}?/man[18]/!) {
	    push @main_bin_body, $_;
	} elsif ($srcname eq $libname and m!^\%\{?_libdir\}?/lib\%\{name\}\.so\.\d*\*\s*$!) {
	    push @shared_lib_bad_glob_body, $_;
	# %{_libdir}/girepository-1.0/Fep*.typelib
	} elsif (m!^\%\{?_libdir\}?/girepository-1\.0/.*\.typelib!) {
	    push @shared_lib_gir_body, $_;
	    die "Shared Libs Policy converter: Unsupported(gir): $_";
	} else {
	    warn 'Shared Libs Policy converter state: libname=',$libname, ' solib_major=',$solib_major,"\n";
	    die "Shared Libs Policy converter: Unsupported: $_";
	}
    };
    while ($shared_lib_body[$#shared_lib_body] eq "\n") {push @endspace_body, pop @shared_lib_body};
    #warn 'DEBUG: soname_glob_main=',$soname_glob_main,' soname_glob_extra=',$soname_glob_extra,"\n";
    push @shared_lib_body, '%_libdir/lib'.$libname.'.so.'.$solib_major."\n" unless $soname_glob_main;
    push @shared_lib_body, '%_libdir/lib'.$libname.'.so.'.$solib_major.'.*'."\n" unless $soname_glob_extra;
    #warn 'DEBUG: shared_lib_body=',@shared_lib_body;
    my $shlib_pkgname='-n '.$sharedlibname;
    if (@main_bin_body) {
	$donor_sec->set_body([
	    # old pkg
	    $donor_head,@main_doc_body,@main_bin_body,@endspace_body,
	    # shlib pkg
	    '%files '.$shlib_pkgname."\n",@main_doc_body,@shared_lib_body,@endspace_body
			     ]);
    } else {
	$donor_sec->set_body([$donor_head,@main_doc_body,@shared_lib_body,@endspace_body]);
    }
    my $add_shlib_pkg=1;
    if (@main_bin_body==0 and $libpkg ne '') { ## if srcname ne $libpkg
	$spec->rename_package($libpkg,$sharedlibname);
	$add_shlib_pkg=0;
    } elsif (@main_bin_body==0) {
	$donor_sec->set_raw_package('-n '.$sharedlibname);
    } elsif (@main_doc_body and $libpkg eq '') {
	# TODO: implement adding utils subpackage
	die 'Oops! SharedLibsPolicy converter: unsupported (yet) package: main %files has the same name as shared lib package, but contains (s)bin/man dir entries';
    }
    if ($add_shlib_pkg) {
	my $descr_sec=$spec->get_section('description',$libpkg);
	my @descr_body=@{$descr_sec->get_bodyref()};
	# TODO __get_header_idx to be used someday
	shift @descr_body;
	while ($descr_body[$#descr_body] eq "\n") {pop @descr_body};
	$descr_sec->push_body(q!
%package -n !.$sharedlibname.q!
Summary:        Shared library for the %name library
Group:          System/Libraries

%description -n !.$sharedlibname."\n",@descr_body,q!
This package contains the shared library.
!);
    }

    # fix requires in subpackages
    if ($libpkg eq '') {
	foreach my $sec ($spec->get_sections) {
	    next if $sec->get_type ne 'package';
	    next if $sec->is_main_section;
	    # if if (@main_bin_body==0)
	    $sec->map_body(
	    sub{
		#Requires:       %{name} = %{version}-%{release}
		#Requires:       %{name}%{?_isa} = %{version}-%{release}
		#warn "R0:",$_ if /^Requires/;
		s/^(Requires:\s*)(?:$srcname|\%\{?name\}?\%\{\?_isa\}|\%\{?name\}?)(\s+=\s+)(?:\%\{\?epoch:\%epoch:\})?(?:\%\{?epoch\}?)?\%\{?version\}?-\%\{?release\}?(\s*)$/$1$sharedlibname$2%EVR$3/;
		#warn "R1:",$_ if /^Requires/;
	    });
	}
    } else {
	die 'NOT IMPLEMENTED';
	#check for $normalized_name in requires
    }
    # devel;
    my $develsec=$spec->get_section('package','devel');
    my $develshlibpkg='-n lib'.$libname.'-devel';
    if (substr($srcname,0,3) ne 'lib' and $develsec and not $spec->get_section('package',$develshlibpkg)) {
	$develsec->push_body('Provides: %name-devel = %EVR'."\n");
	$spec->rename_package('devel',$develshlibpkg);
    }

    $spec->_reset_speclist;

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

# TODO: enable by opt if shared libs policy is used
sub opt_handler_unpackaged_files_terminate_build {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $mainsec=$spec->main_section;
    $mainsec->exclude_body('_unpackaged_files_terminate_build');
    $mainsec->unshift_body('%define _unpackaged_files_terminate_build 1'."\n");
}

1;


