package RPM::Source::Convert::Plugins::rpm2rpm404;
use strict;
use warnings;
#use Carp;
use File::Basename;

use RPM::Source::Tools::Dependency::Filter;
use RPM::Source::Transformation::Factory;
use RPM::Source::Transformation::Factory::DependencyFilter;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Convert::Plugins::SRPMReset;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;
our $rpm404=1;
# come from Fedora2ALT
our $opt_rpm404_touch_ghosts=1;
# due to rpm404
#our $opt_force_ascii=1;
my $opt_force_ascii=$rpm404;
my $opt_ignore_autopatch_scm;

my $__scm_username='rpmbuild';
my $__scm_usermail='<rpmbuild>';
my $__scm_author=$__scm_username.' '.$__scm_usermail;


our @LONGOPT=(
    'rpm404!' => \$rpm404,
    'touch-ghosts!' => \$opt_rpm404_touch_ghosts,
#    'force-ascii!' => \$opt_force_ascii,
    );

sub print_longopt {
    print "rpm 2 rpm 4.0.4 conversion  options:\n";
    print "	--no-rpm404  disable conversion to rpm 4.0.4 altlinux format\n";
    print "	--no-touch-ghosts  disable touch in %install %ghost-s in %files\n";
#    print "	--no-force-ascii  do not drop non-ascii characters from description\n";
}

sub register_messages {
    my ($self,$extopt)=@_;
    ### TODO: use some fixed keys hash here
    RPM::Source::Transformation::Component::add_opt($extopt, {
#	-rpm404 => 0,
	});
}

&RPM::Source::Transformation::Factory::DependencyFilter::register_filter(
    -name => 'drop release in version',
    -description => 'drop release in version in Buildreq/Req tags, like Req: foo > 2.0-2',
    -class => 'rpm2rpm404::DependencyFilter::RPM404DropReleaseInVersion',
    -tags => SPEC_TAG_SET_REQUIRES,
    -priority => FILTER_PRIORITY_PREDISTROMAP,
);

sub transformations {
    return unless $rpm404;
    my @transformations=(
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_REINIT,
	    -name => 'convert to rpm404: reinit',
	    -opt_handler_sub=>\&opt_handler_rpm404_reinit),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_POST_REINIT,
	    -name => 'convert to rpm404: post reinit',
	    -opt_handler_sub=>\&opt_handler_rpm404_post_reinit),
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'convert to rpm404: main',
	    -opt_handler_sub=>\&opt_handler_rpm404_main),
	);
    push @transformations,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_CONVERT,
	    -name => 'rpm404_touch_ghosts',
	    -opt_handler_sub=>\&opt_handler_rpm404_touch_ghosts)
	if $opt_rpm404_touch_ghosts;
    return @transformations;
}

sub lua_gsubst {
    #my $verbose=9;
    my ($instring, $pattern, $subst)=@_;
    $pattern=~s/\[%p\%s\]/[:;,=_`'"#\\s]/g;
    $pattern=~s/\%s/\\s/g;
    #$pattern=~s/\%p/(?:,|\.|-|_|=|!|\||\\|<|>|`|'|"|;|:|~)/g;
    my $outstring= $instring=~s/$pattern/$subst/gr;
    print STDERR 'rpm2rpm404:DEBUG: lua_gsubst: instring=',$instring,' outstring=',$outstring,' repattern=',$pattern,' subst=',$subst,"\n" if $verbose > 4;
    return $outstring;
}

sub lua_rpmexpand {
    #my $verbose=9;
    my ($macros, $instring)=@_;
    my $outstring=$macros->macro_subst($instring);
    print STDERR 'rpm2rpm404:DEBUG: lua_rpmexpand: instring=',$instring,' outstring=',$outstring,"\n" if $verbose > 4;
    return $outstring;
}


sub lua_otherfunc {
    #my $verbose=9;
    my ($instring, $func)=@_;
    my $outstring=$instring;
    if ($func) {
	if ($func eq 'string.lower(') {
	    $outstring=lc($instring);
	} else {
	    die "rpm404: lua_otherfunc: unsupported func ", $func;
	}
    }
    print STDERR 'rpm2rpm404:DEBUG: lua_otherfunc: instring=',$instring,' outstring=',$outstring,' func=',$func,"\n" if $verbose > 4;
    return $outstring;
}


sub lua_gsubst3 {
    my ($macros, $instring, $pattern, $subst,$func)=@_;
    my $outstring=&lua_otherfunc(&lua_gsubst(&lua_rpmexpand($macros,$instring),$pattern,$subst),$func);
    return $outstring;
}

sub lua_gsubst6 {
    my ($macros, $instring, $pattern1, $subst1, $pattern2, $subst2, $func)=@_;
    my $outstring=&lua_otherfunc(&lua_gsubst(&lua_gsubst(&lua_rpmexpand($macros,$instring),$pattern1,$subst1),$pattern2,$subst2),$func);
    return $outstring;
}

sub opt_handler_rpm404_reinit {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    #%{lua:t=string.gsub(rpm.expand("%{foundry} %{fontfamily}"), "[%p%s]+", "_");print(t)}
    #%{lua:t=string.gsub(rpm.expand("%{fontfamily}"), " SIL$", ""); t=string.gsub(t, "[%p%s]+", ""); print(string.lower(t))}
    my $macros=$spec->macros;
    my $mainsec=$spec->main_section;
    my $is_reset_sources;
    $mainsec->map_body(
	sub{
	    $is_reset_sources=1 if s/\%\{lua:t=string\.gsub\(rpm\.expand\("([^"]*)"\),\s*"([^"]*)",\s*"([^"]*)"\);\s*print\((string\.lower\()?t\)?\)\}/&lua_gsubst3($macros,$1,$2,$3,$4)/e;
	    $is_reset_sources=1 if s/\%\{lua:t=string\.gsub\(rpm\.expand\("([^"]*)"\),\s*"([^"]*)",\s*"([^"]*)"\);\s*t=string\.gsub\(t,\s*"([^"]*)",\s*"([^"]*)"\);\s*print\((string\.lower\()?t\)?\)\}/&lua_gsubst6($macros,$1,$2,$3,$4,$5,$6)/e;
	});
    $opt->{-srpm_reset_sources_after_reinit}=1 if $is_reset_sources;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_rpm404_post_reinit {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $is_reset_macros;
    foreach my $sec ($spec->get_sections()) {
	my $type=$sec->get_type;
	next if $type eq 'changelog';
	$is_reset_macros=1 if &alt_reinit_convert_expand_multiline($sec);
    }
    if ($is_reset_macros) {
	$opt->{-srpm_reset_macros_after_reinit}=1;
	#$opt->{-srpm_reset_sources_after_reinit}=1;
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __scm_br {
    my ($scm_type)=@_;
    return '' if !$scm_type or $scm_type eq 'patch';
    return '/usr/bin/gendiff' if $scm_type eq 'gendiff';
    return '/usr/bin/quilt' if $scm_type eq 'quilt';
    # Mercurial (aka hg)
    return '/usr/bin/hg' if $scm_type eq 'hg';
    # Git / Git am
    return '/usr/bin/git' if $scm_type eq 'git' or $scm_type eq 'git_am';
    # Bzr
    return '/usr/bin/bzr' if $scm_type eq 'bzr';
    die "autosetup: unsupported scm ",$scm_type;
}

sub __scm_setup {
    my ($scm_type, $q_arg)=@_;
    return '' if !$scm_type or $scm_type eq 'patch' or $scm_type eq 'gendiff' or $scm_type eq 'quilt';
    # Mercurial (aka hg)
    return 'hg init '.$q_arg.' .
hg add '.$q_arg.' .
hg commit '.$q_arg.' --user "'.$__scm_author.'" -m "%{NAME}-%{VERSION} base"
' if $scm_type eq 'hg';
    # Git / Git am
    return 'git init '.$q_arg.'
git config user.name "'.$__scm_username.'"
git config user.email "'.$__scm_usermail.'"
git config gc.auto 0
git add --force .
git commit '.$q_arg.' --allow-empty -a --author "'.$__scm_author.'" -m "%{NAME}-%{VERSION} base"
' if $scm_type eq 'git' or $scm_type eq 'git_am';
    # Bzr
    return 'bzr init '.$q_arg.'
bzr whoami --branch "'.$__scm_author.'"
bzr add .
bzr commit '.$q_arg.' -m "%{NAME}-%{VERSION} base"
' if $scm_type eq 'bzr';
    die "autosetup: unsupported scm ",$scm_type;
}

# Plain patch (-m is unused)
sub __scm_apply_patch {
    my ($q_arg,$p_arg,$m_arg,$file,$num)=@_;
    return 'patch'.($p_arg ? ' -p'.$p_arg:'').($q_arg ? ' -s':'')."\n";
}

# Plain patch with backups for gendiff
sub __scm_apply_gendiff {
    my ($q_arg,$p_arg,$m_arg,$file,$num)=@_;
    return 'patch'.($p_arg ? ' -p'.$p_arg:'').($q_arg ? ' -s':'').' -b --suffix ".'.$num.'"'."\n";
}

# Mercurial (aka hg)
sub __scm_apply_hg {
    my ($q_arg,$p_arg,$m_arg,$file,$num)=@_;
    return 'hg import - '.($p_arg ? ' -p'.$p_arg:'').' '.$q_arg.' -m '.$m_arg.' --user "'.$__scm_author.'"'."\n";
}

# Git
sub __scm_apply_git {
    my ($q_arg,$p_arg,$m_arg,$file,$num)=@_;
    return 'git apply --index --reject '.($p_arg ? ' -p'.$p_arg:'').' -'."\n".
	'git commit '.$q_arg.' -m '.$m_arg.' --author "'.$__scm_author.'"'."\n";
}

sub __scm_apply_git_am {
    my ($q_arg,$p_arg,$m_arg,$file,$num)=@_;
    return 'git am --reject '.$q_arg.($p_arg ? ' -p'.$p_arg:'')."\n";
}

# Quilt
sub __scm_apply_quilt {
    my ($q_arg,$p_arg,$m_arg,$file,$num)=@_;
    return 'quilt import '.($p_arg ? ' -p'.$p_arg:'').$file.' && quilt push '.$q_arg."\n";
}

# Bzr
# bzr doesn't seem to have its own command to apply patches?
sub __scm_apply_bzr {
    my ($q_arg,$p_arg,$m_arg,$file,$num)=@_;
    return 'patch'.($p_arg ? ' -p'.$p_arg:'').($q_arg ? ' -s':'')."\n".
    'bzr commit '.$q_arg.' -m '.$m_arg."\n";
}

my $scm_apply_handlers={
    'patch' => \&__scm_apply_patch,
    'gendiff' => \&__scm_apply_gendiff,
    'hg' => \&__scm_apply_hg,
    'git' => \&__scm_apply_git,
    'git_am' => \&__scm_apply_git_am,
    'quilt' => \&__scm_apply_quilt,
    'bzr' => \&__scm_apply_bzr,
};

# Single patch application
sub __scm_apply {
    my ($scm_type, $q_arg,$p_arg,$m_arg,$file,$num)=@_;
    my $handler=$scm_apply_handlers->{$scm_type};
    die 'Fatal: %autopatch: no handler for -S ', $scm_type unless $handler;
    my $apply_string=&$handler($q_arg,$p_arg,$m_arg,$file,$num);
    my $catcmd='cat';
    if ($file=~/\.(bz2|xz|gz|lzma)$/) {
	if ($1 eq 'gz') {
	    $catcmd='zcat';
	} elsif ($1 eq 'bz2') {
	    $catcmd='bzcat';
	} elsif ($1 eq 'xz') {
	    $catcmd='xzcat';
	} elsif ($1 eq 'lzma') {
	    $catcmd='lzcat';
	}
    }
    return $catcmd.' '.$file.' | '.$apply_string;
}

sub __patch_apply {
    my ($scm_type,$q_arg,$p_arg,$m_arg,$file,$num)=@_;
    if ($opt_ignore_autopatch_scm or not $scm_type) {
	return &__generic_patch_apply($scm_type,$q_arg,$p_arg,$m_arg,$file,$num);
    } else {
	return &__scm_apply($scm_type,$q_arg,$p_arg,$m_arg,$file,$num);
    }
}

sub __generic_patch_apply {
    my ($scm_type,$q_arg,$p_arg,$m_arg,$file,$num)=@_;
    my $patcharg_p='';
    if ($p_arg) {
	$patcharg_p=' -p'.$p_arg;
    } elsif (&__check_patch_if_git_diff($file)) {
	$patcharg_p=' -p1';
    #} else {
    #	$patcharg_p=' -p1';
    }
    return '%patch'.$num.$patcharg_p."\n";
}

sub __autopatch_string {
    my ($spec,$q_arg,$p_arg,$scm_arg,$min_arg,$max_arg)=@_;
    if ($verbose and $scm_arg and $opt_ignore_autopatch_scm) {
	warn "rpm2rpm404: INFO: autosetup/autopatch option -S $scm_arg: ignored due to opt_ignore_autopatch_scm.\n";
    }
    my $apply_patches_string='';
    my $PCHREF=$spec->raw_patchlist_ref;
    my @PCHNUM;
    $min_arg//=0;
    $max_arg//=scalar @$PCHREF;
    for (my $ii=$min_arg; $ii<$max_arg;$ii++) {
	if ($PCHREF->[$ii]) {
	    push @PCHNUM, $ii;
	}
    }
    my $patchfileprefix='%_sourcedir/';
    my $PATCHCONDCONTEXT=$spec->raw_patchcondcontext_ref;
    foreach my $patchnum (@PCHNUM) {
	my ($patch_pre, $patch_post)=('','');
	my $this_patch_condcontext=$PATCHCONDCONTEXT->[$patchnum];
	if ($this_patch_condcontext) {
	    my @condcontext=@$this_patch_condcontext;
	    $patch_pre=join('',@condcontext);
	    while (@condcontext>0) {
		my $cond=pop @condcontext;
		$patch_post .= '%endif'."\n" if $cond!~/^\%else/;
	    }
	}
	my $patchbasename=basename($PCHREF->[$patchnum]);
	my $patchfile=$patchfileprefix.$patchbasename;
	my $apply_patch_string=&__patch_apply($scm_arg,$q_arg,$p_arg,$patchbasename,$patchfile,$patchnum);
	$apply_patches_string .=$patch_pre.$apply_patch_string.$patch_post;
    }
    return $apply_patches_string;
}

sub opt_handler_rpm404_main {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    &alt_generic_rpm404_check_ahead($spec,$spec->main_section);
    &alt_generic_rpm404_single_f_flag_in_files($spec);
    #my $mainsec=$spec->main_section;
    #print STDERR 'rpm2rpm404: DEBUG: entered opt_handler_rpm404_main verbose=',$verbose,"\n" if $verbose>3;
    my $macros=$spec->macros;
    foreach my $section ($spec->get_sections()) {
	my $t = $section->get_type();
	print STDERR 'rpm2rpm404: processing %',$t,' ',$section->get_raw_package,"\n" if $verbose>2;
	if ($t eq 'package') {
	    # encoding mismatch. alt assume 8 bit
	    # TODO: detect utf8?
	    #$section->exclude_body('^Summary\(de\):');
	    &alt_generic_cleanup_headers($section);
	} elsif ($t eq 'description') {
	    my $l=$section->get_flag('-l');
	    #$section->delete() if $l and $l=~/^(?:de|fr)$/;
	    $section->apply_body(
		sub {
		    my $line=shift;
		    # found in golang-github-influxdb-influxdb
		    # %{%description}
		    if ($line=~/^\%\{\%description\}/) {
			my $desc_body_ref=$spec->get_section('description','')->get_bodyref;
			my @description;
			my @in=@$desc_body_ref;
			shift @in; # kill header
			while (@in) {
			    my $desc_line=shift @in;
			    last if $desc_line=~/^\s*(?:#|$)/ and @description or $desc_line=~/^\s*\%(?:if|else|endif)/;
			    push @description, $desc_line;
			}
			return @description;
		    }
		    $line=&__kill_non_ascii($line) unless $l;
		    return $line;
		});
	} elsif ($t eq 'prep') {
		#$section->subst_body_if(qr'\s-F(?=\d)',' -F ',qr'^\s*\%patch(?:\d|\s)');
		$section->map_body(sub {
		    my $ii;
		    # rpm404 compat: glues line to %patch: %build
		    s,(?<=\s)-b(\S+),-b $1,g if /^\s*\%patch/;
		    # rpm404 compat
		    s,\%setup0\b,%setup,;

		    if (s/(?<!\%)\%apply_patches/#%%apply_patches/) {
			my $patchlist=$spec->raw_patchlist_ref;
			for (my $i=0; $i <@$patchlist;$i++) {
			    next unless $patchlist->[$i];
			    $_.='%patch'.$i.' -p1'."\n";
			}
		    }

		    if (s/^(.*)(?<!%)\%\{?auto(setup|patch)\}?\s*//) {
			my $autoprefix=$1;
			my $autotype=$2;
			# default behavoiur, unless %autosetup -N
			my $autopatch_after_autosetup=1;
			my @args=Text::ParseWords::shellwords($_);
			my @setupargs;
			my $prev_arg='';
			my ($v_arg,$p_arg,$scm_arg,$min_arg,$max_arg);
			while (@args) {
			    my $arg=shift @args;
			    if ($arg eq '-n' or $arg eq '-a' or $arg eq '-b') {
				push @setupargs, $arg, shift @args;
			    } elsif ($arg eq '-N') {
				$autopatch_after_autosetup=0;
			    } elsif ($arg eq '-v') {
				my $v_arg='-v';
			    } elsif ($arg =~ s/^-S//) {
				$arg=shift @args if $arg eq '';
				$scm_arg=$arg;
			    } elsif ($arg =~ s/^-m//) {
				$arg=shift @args if $arg eq '';
				$min_arg=$arg;
				warn 'rpm404:autosetup: strange -m ',$arg if $autoprefix eq 'setup';
			    } elsif ($arg =~ s/^-M//) {
				$arg=shift @args if $arg eq '';
				$max_arg=$arg;
				warn 'rpm404:autosetup: strange -M ',$arg if $autoprefix eq 'setup';
			    } elsif ($arg eq '-p') {
				$p_arg=shift @args;
			    } elsif ($arg =~ s/^-p//) {
				$p_arg=$arg;
			    } elsif ($arg eq '-D' or $arg eq '-T' or $arg eq '-q' or $arg eq '-c') {
				push @setupargs, $arg;
			    } elsif ($prev_arg eq '-c') {
				push @setupargs, '-n', $arg;
			    } else {
				push @setupargs, $arg;
			    }
			    $prev_arg=$arg;
			}
			$spec->main_section->unshift_body('BuildRequires: '.&__scm_br($scm_arg)."\n") if $scm_arg and not $opt_ignore_autopatch_scm;
			unshift @setupargs, '-q' unless $v_arg;
			my $setupstring=join(' ','%setup',@setupargs)."\n";
			$setupstring='' if $autotype eq 'patch';
			my $scm_setup_string='';
			$scm_setup_string=&__scm_setup($scm_arg,($v_arg? '':'-q')) unless $opt_ignore_autopatch_scm;
			my $apply_patches_string='';
			if ($autotype eq 'patch' or $autopatch_after_autosetup) {
			    print STDERR 'DEBUG: generating autopatch: autotype=',$autotype,' after=',$autopatch_after_autosetup,' p=',($p_arg ? $p_arg: '<none>'),' scm=',($scm_arg ? $scm_arg: '<none>'),"\n" if $verbose;
			    $apply_patches_string=&__autopatch_string($spec,($v_arg? '':'-q'),$p_arg,$scm_arg,$min_arg,$max_arg);
			}
			warn "INFO: auto$autotype: ----\n$setupstring$scm_setup_string$apply_patches_string\n----- END ----\n" if $verbose;
			$_=$autoprefix.$setupstring.$scm_setup_string.$apply_patches_string."\n";
			return;
		    }
		    if (/(?<!%)\%\{(?:sources|patches)\}/) {
			my $SRCREF=$spec->raw_sourcelist_ref;
			my $PCHREF=$spec->raw_patchlist_ref;
			my @SRCNUM;
			for ($ii=0; $ii<@$SRCREF;$ii++) {
			    push @SRCNUM, $ii if $SRCREF->[$ii];
			}
			my @PCHNUM;
			for ($ii=0; $ii<@$PCHREF;$ii++) {
			    push @PCHNUM, $ii if $PCHREF->[$ii];
			}
			my $sourcesstring=join(' ',map {'%{SOURCE'.$_.'}'} @SRCNUM);
			my $patchesstring=join(' ',map {'%{PATCH'.$_.'}'} @PCHNUM);
			s/(?<!%)\%\{sources\}/$sourcesstring/g;
			s/(?<!%)\%\{patches\}/$patchesstring/g;
		    }
			       });
	} elsif ($t eq 'files' or $t eq '_font_pkg') {
	    $section->map_body(
	sub {

	    # rpm404
	    # %caps(cap_ipc_lock=ep)
	    s/\%caps\([^\)]+\)//g; # not supported in alt rpm404

	    # not supported in rpm 404
	    # beware of prefix!
	    #%{!?_licensedir: %doc %{_pkgdocdir}/LICENSE}
	    #%{?_licensedir: %license LICENSE}
	    if (m/^(.*)\%(?:license|copying)(\s+)(\S.*)$/) {
		my ($prefix,$space,$suffix)=($1,$2,$3);
		my $expanded_string=$macros->macro_subst($suffix);
		#error: Can't mix special %doc with other forms: /usr/share/doc/slv2/COPYING
		# but we should allow %license %name/GPL
		if (substr($expanded_string,0,1) eq '/') { # was: $suffix=~m!^(?:/|\%)!
		    warn 'rpm2rpm404: INFO: ignored %license (Can\'t mix special %doc with other forms) in ',$suffix if $verbose;
		    $_=$prefix ? $prefix.$space.$suffix."\n" : $suffix."\n";
		} else {
		    $_=$prefix.'%doc --no-dereference'.$space.$suffix."\n";
		}
	    }
	    # {,} in %files, one-liners
	    if (m!^\s*\%\{?_libdir\}?/lib(.*)\.so\.\%\{?major\}?\{,\.\*\}\s*$!) {
		# %{_libdir}/lib%{name}.so.%{major}{,.*} -> not for rpm404
		$_='%{_libdir}/lib'.$1.'.so.%{major}*'."\n";
	    } elsif (m!\.\{py,pyc,pyo\}\s*$!) {
		# {,,} -> not for rpm404
		s!\.\{py,pyc,pyo\}(\s*)$!.py*$1!;
	    } elsif (m!^\s*\%doc\s*$!) {
		# %doc w/o args
		$_='%doc %_docdir/%name'."\n";
	    }

	    # rpm404 compat: {,} in %files, like
	    # like %{_ttffontdir}/*/CreteRound-{Italic,Regular}.otf
	    my $line=$_;
	    if (s/^(.*?)(?<!%)\{//) {
		my $prefix=$1;
		my $body='';
		while (s/^([^%\}]+|\%\{[^\}]+\})//) {
		    $body .= $1;
		}
		if (not s/\}//) {
		    $_=$line;
		    warn "rpm2rpm404: %files: can't unwrap line: $_";
		} else {
		    my $suffix=$_;
		    my @body=split(',',$macros->macro_subst($body));
		    $_=join('',map {$prefix.$_.$suffix} @body);
		}
	    }
			       });
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __check_patch_if_git_diff {
    my ($patchfile)=@_;
    return if ! -e $patchfile;
    open (RPMPATCH, '<', $patchfile) || die "can't open $patchfile: $!";
    local $_;
    my $is_git=0;
    while (<RPMPATCH>) {
	$is_git=1 if m/^--- a\//;
    }
    close (RPMPATCH);
    return $is_git;
}

sub alt_reinit_convert_expand_multiline {
    my ($section)=@_;
    my @out;
    # %define _desc %{expand ... }
    my $expand_multiline_found=0;
    my $is_prev_line_expand_multiline=0;
    local $_;
    #print STDERR 'rpm2rpm404: DEBUG: entered ag_convert_expand_multiline verbose=',$verbose,"\n" if $verbose>3;
    foreach (@{$section->get_bodyref()}) {
	#print STDERR 'rpm2rpm404: DEBUG: ag_convert_expand_multiline: M=',$is_prev_line_multiline,' E=',$is_prev_line_expand_multiline,' in: ',$_ if $verbose>8;
	if ($is_prev_line_expand_multiline) {
	    print STDERR 'rpm2rpm404: reinit in %{expand: multiline: ',$_ if $verbose>3;
	    my $is_expand_multiline_closed=0;
	    my $bracket_count=0;
	    #$is_expand_multiline_closed=1 if (s/\}//);
	    my $len=length($_);
	    for (my $i=0; $i<$len; $i++) {
		my $char_i=substr($_,$i,1);
		if ($char_i eq '{') {
		    $bracket_count--;
		} elsif ($char_i eq '}') {
		    $bracket_count++;
		}
	    }
	    if ($bracket_count>0) {
		$is_expand_multiline_closed=1;
		# last found \}; better be balanced }, but it is more complicated
		s/^(.*)\}/$1/;
	    }
	    if ($is_expand_multiline_closed) {
		print STDERR 'rpm2rpm404: %{expand: multiline: ended. }',$_ if $verbose>3;
		$is_prev_line_expand_multiline=0;
	    } else {
		# some mutant %{expands also have \ inside; append \\ only if needed
		s/(?<!\\)\n$/\\\n/;
	    }
	    push @out, $_;
	# not:
	#%global fontdescription0 %{expand:%{common_description}
	} elsif (/\%\{expand:.*\}/ and not /\%\{expand:\s*\%\{[\w_\d]+\}\s*$/) {
	    # one-line expand, do nothing ...
	    push @out, $_;
	} elsif (s/\%\{expand://) {
	    print STDERR 'rpm2rpm404: found %{expand: multiline: ',$_ if $verbose>3;
	    $expand_multiline_found=1;
	    $is_prev_line_expand_multiline=1;
	    s/\n$/\\\n/;
	    push @out, $_;
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
    return $expand_multiline_found;
}

# TODO: remove multiline: moved to alt_reinit_convert_expand_multiline
sub alt_generic_cleanup_headers {
    my ($section)=@_;
    my @out;
    # traditional multiline with \<eol>(\\\n).
    my $is_prev_line_multiline=0;
    local $_;
    #print STDERR 'rpm2rpm404: DEBUG: entered ag_cleanup_headers verbose=',$verbose,"\n" if $verbose>3;
    foreach (@{$section->get_bodyref()}) {
	#print STDERR 'rpm2rpm404: DEBUG: ag_cleanup_headers: M=',$is_prev_line_multiline,' in: ',$_ if $verbose>8;
	# no %{URL} in rpm404
	if (/\%\{?[Uu][Rr][Ll]\}?\b/) {
	    my $url=$section->get_tag('URL',RAW=>1);
	    # not s,\%\{?[Uu][Rr][Ll]\}?\b,$url, !!! -- bad, \b matches before \} (maybe due to \}? )
	    s,\%\{?[Uu][Rr][Ll]\}?,$url,;
	}
	if (/^summary:/i) {
	    push @out, &__kill_non_ascii($_);
	} elsif (/^\s*#/i) {
	    #in comment: conakry-fonts-20070829-4.fc15/*.spec does not look like a spec file
	    push @out, &__kill_non_ascii($_);
	## alt has VCS now too.
	#} elsif (/^(?:VCS):/i) {
	#    push @out, '#'.$_;
	} elsif (/^(?:Supplements|Enhances):/) {
	    print STDERR 'info: rpm2rpm404: dropped ',$_;
	    # exclude line
	} elsif (/\\\n$/ or $is_prev_line_multiline) {
	    # multiline macro for description: fonts-otf-beteckna
	    push @out, &__kill_non_ascii($_);
	} elsif (/^\%global\s+fontsummary/) {
	    # gfs-goschen-fonts summary
	    push @out, &__kill_non_ascii($_);
	} else {
	    push @out, $_;
	}
	$is_prev_line_multiline= /\\\n$/;
    }
    $section->set_body(\@out);
}

sub __kill_non_ascii {
    my ($line)=@_;
    return $line if not $opt_force_ascii;
    local $_=$line;
    # some distro use iso or utf8 in description. ALT Linux use ascii only. :(
    #tr//e/;
    tr/ޜ\x98\x99/-.....eooeaiaoucououaaidieaayvnieouabEOOEAIAOUCOUOUAAIDIEAAYVNIEOUAB''''/;
    # exterminatus of the rest of the non-ascii characters
    s/[\x80-\xff]/./g;
    if ($verbose and $line ne $_) {
	print STDERR "info: killed non-ascii for rpm404 in [$line]\n";
    }
    return $_;
}

# is macro %name used in spec before Name:,Version: tags
# it is not allowed in alt
sub alt_generic_rpm404_check_ahead {
    my ($spec,$section)=@_;
    my @out;
    my ($is_name_passed,$is_name_ahead);
    my ($is_version_passed,$is_version_ahead);
    my ($is_release_passed,$is_release_ahead);
    foreach (@{$section->get_bodyref()}) {
	m/^\s*name:/i and $is_name_passed=1; #and print STDERR "passed $_\n";
	m/^\s*version:/i and $is_version_passed=1; #and print STDERR "passed $_\n";
	m/^\s*release:/i and $is_release_passed=1; #and print STDERR "passed $_\n";
	$is_name_ahead=1 if not $is_name_passed and (m/\%\{name\}/ or m/\%name(?:[^\w\d_]|$)/);
	$is_version_ahead=1 if not $is_version_passed and (m/\%\{version\}/ or m/\%version(?:[^\w\d_]|$)/);
	$is_release_ahead=1 if not $is_release_passed and (m/\%\{release\}/ or m/\%release(?:[^\w\d_]|$)/);
	last if $is_name_passed and $is_version_passed and $is_release_passed;
    }
    if ($is_name_ahead or $is_version_ahead or $is_release_ahead) {
	my @predefs;
	push @predefs, '%%name' if $is_name_ahead;
	push @predefs, '%%version' if $is_version_ahead;
	push @predefs, '%%release' if $is_release_ahead;
	my $predefs_title=join(' and ',@predefs);
	my $main_section=$spec->main_section;
	$section->unshift_body(
	    "# $predefs_title is ahead of its definition. Predefining for rpm 4.0 compatibility.\n".
	    ($is_name_ahead ? '%define name '.$main_section->get_tag('Name')."\n" :'').
	    ($is_version_ahead ? '%define version '.$main_section->get_tag('Version')."\n" :'').
	    ($is_release_ahead ? '%define release '.$main_section->get_tag('Release')."\n" :'')
	    );
	$predefs_title=~s/\%\%/%/g;
	print STDERR "INFO: $predefs_title is ahead of its definition - fixed.\n";
    }
}

sub alt_generic_rpm404_single_f_flag_in_files {
    my ($spec)=@_;
    my $installsec=$spec->get_section('install');
    ### TODO
    my %base_filelists;
    # TODO - merge body here
    # my %base_sections;
    my $prev_section_found=0;
    foreach my $section ($spec->get_sections()) {
	my $t = $section->get_type();
	next if $t ne 'files';
	my @f_flags=$section->get_flag('-f');
	next unless @f_flags;
	# hack around old bug in get_glag
	next if @f_flags==1 and not defined($f_flags[0]);
	my $cname=$section->get_canonical_package();
	my $base_filelist=$base_filelists{$cname};
	if (defined $base_filelist) {
	    $prev_section_found=1;
	} else {
	    $base_filelist=shift @f_flags;
	    $base_filelists{$cname}=$base_filelist;
	    # $base_sections{$cname}=$section;
	    $prev_section_found=0;
	}
	if (@f_flags>0) {
	    my $cname_verbose=$cname ? $cname : '<main>';
	    unless ($installsec) {
		print STDERR "WARN: extra -f flags but no \%install section :-(\n";
	    } else {
		my $headlineref=$section->get_headline_ref();
		die "Oops! can't be: no neadline" unless $headlineref;
		foreach my $f_file (@f_flags) {
		    unless (defined $f_file) {
			die 'rpm2rpm404: merge multiple %files: internal error: undefined argumant to -f in %files for ',$cname_verbose. "\n", 'section: -f values are ',join(':',map {defined($_)? ($_ ? $_ : '<empty>') : '<undef>'} $section->get_flag('-f')),". body is:\n",$section->describe();
		    }
		    print STDERR "INFO: extra -f flags: merging file list $f_file into $base_filelist\n";
		    $installsec->push_body("# multiple -f flags in \%files for $cname_verbose: merging -f $f_file into -f $base_filelist
cat $f_file >> $base_filelist\n");
		    $$headlineref =~ s/-f\s+\Q$f_file\E//;
		}
		# destroy this section as the same section already exists.
		# see ant for example:
		#%files junit -f .mfiles-junit
		#%files junit -f .mfiles-junit4
		if ($prev_section_found) {
		    print STDERR "INFO: extra %files for $cname_verbose: merged and dropped\n";
		    $$headlineref='';
		}
	    }
	}
    }
}


sub opt_handler_rpm404_touch_ghosts {
    my (undef,$spec,undef,$opt,$resourceobj)=@_;
    my $macros=$spec->macros();
    my (@ghosts,@ghostdirs);
    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	if ($t eq 'files') {
	    $sec->map_body(sub {
		if (/(?<!%)\%ghost\s/) {
		    my $ghost=$_;
		    $ghost =~ s/\s*$//;
		    # %attr, %ghost, %...
		    $ghost =~ s/^.*(?<!\\)\s+//;
		    print STDERR "DEBUG: rpm404_touch_ghosts: found $ghost in $_" if $verbose>1;
		    if ($ghost=~m!/\s*$!) {
			push @ghostdirs, $ghost;
		    } else {
			push @ghosts, $ghost;
		    }
		}
	   });
	}
    }
    if (@ghosts>0 or @ghostdirs>0) {
	my $instsec=$spec->add_section('install');
	$instsec->push_body("\n",q!# touching all ghosts; hack for rpm 4.0.4!,"\n"),
	    $instsec->push_body(q!for rpm404_ghostdir in !.join(' ',@ghostdirs).q!
do
    mkdir -p %buildroot`dirname "$rpm404_ghostdir"`
done!."\n") if @ghostdirs;
	$instsec->push_body(q!for rpm404_ghost in !.join(' ',@ghosts).q!
do
    mkdir -p %buildroot`dirname "$rpm404_ghost"`
    touch %buildroot"$rpm404_ghost"
done!."\n") if @ghosts;
	$instsec->push_body("\n");
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;

package rpm2rpm404::DependencyFilter::RPM404DropReleaseInVersion;
use strict;
use warnings;
our @ISA=q/RPM::Source::Tools::Dependency::Filter/;

sub apply {
    my ($self, $tagobj, $dep) = @_;
    return unless $RPM::Source::Convert::Plugins::rpm2rpm404::rpm404;
    my $pkgversion=$dep->{-version};
    if ($pkgversion and $pkgversion=~m!-.+$! and $pkgversion!~m/-\%\{?release\}?$/
	# not = %(pkg-config --modversion libpari23 2>/dev/null || echo 0)
	and $pkgversion!~m/^\%\(/
	) {
	$pkgversion=~s!-.+$!!;
	$dep->{-version}=$pkgversion;
    }
    return $dep;
}

1;

__END__
