package RPM::Source::Convert::Tools::LuaFedoraCommon;

use strict;
use warnings;
use Carp;

our $verbose=0;

sub __safeget_macrostring {
    my ($macros, $macroname,$defaults)=@_;
    if ($macros->get_raw($macroname)) {
	return '%{'.$macroname.'}';
    } else {
	return $defaults;
    }
}

sub wordwrap {
    my $text=shift;
    $text .= "\n";
    $text =~ s,\t,  ,g;
    $text =~ s,\r,\n,g;
    $text =~ s, +\n,\n,g;
    $text =~ s,\n+\n,\n\n,g;
    $text =~ s,^\n,,g;
    $text =~ s,\n( *)[-*—][  ]+,\n$1– ,g;
    my $output = '';
    while ($text =~ s,^([^\n]*\n),,s) {
	my $line = $1;
	my $pos = 0;
	my $advance = '';
	while ($line =~ s,^(\s*[^\s]*\n?),,s) {
	    my $word = $1;
	    last if $word eq '';
	    # TODO: utf8
	    my $wl = length($word);
	    if ($pos == 0) {
		if ($word=~ m,^(\s*– ),) {
		    $advance = $1;
		} elsif ($word=~ m,^(\s*),) {
		    $advance = $1;
		}
		$advance =~ s,– ,  ,;
		$pos += $wl;
	    } elsif (($pos + $wl  < 81) or
		(($pos + $wl == 81) and $word =~ m/\n$/)) {
		$pos += $wl;
	    } else {
		$word = $advance . $word=~s,^%s*,,r;
		$output .= "\n";
		# TODO: utf8
		$pos = length($word);
	    }
	    $output .= $word;
	    if ($pos > 80) {
		$pos = 0;
		if ($word !~ m/\n$/) {
		    $output .= "\n";
		}
	    }
	}
    }
    $output =~ s,\n*$,\n,;
    return $output;
}

1;

__END__
-- https://github.com/rpm-software-management/rpm/issues/566
-- Reformat a text intended to be used used in a package description, removing
-- rpm macro generation artefacts.
-- – remove leading and ending empty lines
-- – trim intermediary empty lines to a single line
-- – fold on spaces
-- Should really be a %%{wordwrap:…} verb
local function wordwrap(text)
  text = rpm.expand(text .. "\n")
  text = string.gsub(text, "\t",              "  ")
  text = string.gsub(text, "\r",              "\n")
  text = string.gsub(text, " +\n",            "\n")
  text = string.gsub(text, "\n+\n",           "\n\n")
  text = string.gsub(text, "^\n",             "")
  text = string.gsub(text, "\n( *)[-*—][  ]+", "\n%1– ")
  output = ""
  for line in string.gmatch(text, "[^\n]*\n") do
    local pos = 0
    local advance = ""
    for word in string.gmatch(line, "%s*[^%s]*\n?") do
      local wl, bad = utf8.len(word)
      if not wl then
        print("%{warn:Invalid UTF-8 sequence detected in:}" ..
              "%{warn:" .. word .. "}" ..
              "%{warn:It may produce unexpected results.}")
        wl = bad
      end
      if (pos == 0) then
        advance, n = string.gsub(word, "^(%s*– ).*", "%1")
        if (n == 0) then
          advance = string.gsub(word, "^(%s*).*", "%1")
        end
        advance = string.gsub(advance, "– ", "  ")
        pos = pos + wl
      elseif  (pos + wl  < 81) or
             ((pos + wl == 81) and string.match(word, "\n$")) then
        pos = pos + wl
      else
        word = advance .. string.gsub(word, "^%s*", "")
        output = output .. "\n"
        pos = utf8.len(word)
      end
      output = output .. word
      if pos > 80 then
        pos = 0
        if not string.match(word, "\n$") then
          output = output .. "\n"
        end
      end
    end
  end
  output = string.gsub(output, "\n*$", "\n")
  return output
end
